@@ -56,7 +56,7 @@ class ProxyManager:
                  test_url: str,
                  proxy_info_dict: Dict[str, List[str]],
                  num_executors=32,
-                 executor_queue_size=1000,
+                 concurrent_trials_per_session=32,
                  verbose=False,
                  new_session_fn=lambda: requests.Session(),):
         """
@@ -81,6 +81,7 @@ class ProxyManager:
 
         # self._executor_running = False
         self._executor_workers = num_executors
+        self._concurrent_trials_per_session = concurrent_trials_per_session
         # self._executor_queue = Queue(executor_queue_size)
         self._executor = ThreadPoolExecutor(max_workers=self._executor_workers)
 
@@ -142,7 +143,7 @@ class ProxyManager:
         async def task_fn():
             success = False
 
-            while not end_event.set() and not success:
+            while not end_event.is_set() and not success:
                 with await self._pool.poll() as proxy:
                     session = self._new_session_fn()
 
@@ -154,15 +155,8 @@ class ProxyManager:
                         result_future = self._loop.run_in_executor(
                             self._executor, functools.partial(fn, session, *args))
 
-                        while not result_future.done():
-                            await asyncio.sleep(0.1)
-                            if end_event.is_set():
-                                break
-
-                        if result_future.cancelled():
-                            break
-
                         result = await result_future
+                        timer.stop()
 
                         if type(result) is tuple:
                             callback_async(*result)
@@ -173,6 +167,7 @@ class ProxyManager:
                         return result
                     except Exception as ex:
                         exception_handle(proxy.info(), ex)
+                        continue
                     finally:
                         proxy.update_response_time(
                             timer.time_elapsed() if success else 999.0)
@@ -182,23 +177,28 @@ class ProxyManager:
                         else:
                             self._on_fail()
 
-                        if self._verbose:
-                            print("success rate:", self.success_rate())
-
         # task = Task(task_fn)
         # self._executor_queue.put(task, True)
 
-        done, pending = await asyncio.wait([asyncio.create_task(task_fn()) for _ in range(8)],
+        futures = [asyncio.create_task(task_fn())
+                   for _ in range(self._concurrent_trials_per_session)]
+        done, pending = await asyncio.wait(futures,
                                            return_when=asyncio.FIRST_COMPLETED)
         end_event.set()
-        for future in done:
+        for future in futures:
             if not future.done():
                 future.cancel()
+
         future = next(iter(done))
 
+        if self._verbose:
+            print("success rate:", self.success_rate())
+
         if wait:
-            # return task.get()
-            return await future
+            try:
+                return await future
+            except Exception as ex:
+                exception_handle(None, ex)
 
     async def start(self):
         await self._pool.start()
